﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

using DevExpress.Xpf.Mvvm;

namespace TestProject
{
    /// <summary>
    /// 제품 리스트 뷰 모델
    /// </summary>
    public class ProductListViewModel : ViewModelBase 
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 제품 컬렉션
        /// </summary>
        private ObservableCollection<Product> productCollection = new ObservableCollection<Product>();

        /// <summary>
        /// 제품 폼 보여주기 명령
        /// </summary>
        private ICommand showProductFormCommand;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 제품 컬렉션 - ProductCollection

        /// <summary>
        /// 제품 컬렉션
        /// </summary>
        public ObservableCollection<Product> ProductCollection
        {
            get
            {
                return this.productCollection;
            }
        }

        #endregion
        #region 제품 폼 보여주기 명령 - ShowProductFormCommand

        /// <summary>
        /// 제품 폼 보여주기 명령
        /// </summary>
        public ICommand ShowProductFormCommand
        {
            get
            {
                if(this.showProductFormCommand == null)
                {
                    this.showProductFormCommand = new DelegateCommand<string>(OnShowProductFormCommandExecute);
                }

                return this.showProductFormCommand;
            }
        }

        #endregion
        #region 대화 상자 서비스 - DialogService

        /// <summary>
        /// 대화 상자 서비스
        /// </summary>
        public IDialogService DialogService
        {
            get
            {
                return GetService<IDialogService>();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ProductListViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public ProductListViewModel() : base()
        {
            this.productCollection.Add(new Product() { ID = 1000, ProductName = "Chang"         , Country = "UK"     , City = "Cowes"         , UnitPrice = 19, Quantity = 10 });
            this.productCollection.Add(new Product() { ID = 1001, ProductName = "Gravad lax"    , Country = "Italy"  , City = "Reggio Emilia" , UnitPrice = 12, Quantity = 16 });
            this.productCollection.Add(new Product() { ID = 1002, ProductName = "Ravioli Angelo", Country = "Brazil" , City = "Rio de Janeiro", UnitPrice = 19, Quantity = 12 });
            this.productCollection.Add(new Product() { ID = 1003, ProductName = "Tarte au sucre", Country = "Germany", City = "QUICK-Stop"    , UnitPrice = 22, Quantity = 50 });
            this.productCollection.Add(new Product() { ID = 1004, ProductName = "Steeleye Stout", Country = "USA"    , City = "Reggio Emilia" , UnitPrice = 18, Quantity = 20 });
            this.productCollection.Add(new Product() { ID = 1005, ProductName = "Pavlova"       , Country = "Austria", City = "Graz"          , UnitPrice = 21, Quantity = 52 });
            this.productCollection.Add(new Product() { ID = 1006, ProductName = "Longlife Tofu" , Country = "USA"    , City = "Boise"         , UnitPrice = 7 , Quantity = 120});
            this.productCollection.Add(new Product() { ID = 1007, ProductName = "Alice Mutton"  , Country = "Mexico" , City = "Mexico D.F."   , UnitPrice = 21, Quantity = 15 });
            this.productCollection.Add(new Product() { ID = 1008, ProductName = "Alice Mutton"  , Country = "Canada" , City = "Tsawwassen"    , UnitPrice = 44, Quantity = 16 });
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 제품 폼 보여주기 명령 실행시 처리하기 - OnShowProductFormCommandExecute(parameter)

        /// <summary>
        /// 제품 폼 보여주기 명령 실행시 처리하기
        /// </summary>
        /// <param name="parameter">매개 변수</param>
        private void OnShowProductFormCommandExecute(string parameter)
        {
            if(string.IsNullOrEmpty(parameter))
            {
                return;
            }

            Product product = new Product() { ProductName = parameter };

            UICommand addProductUICommand = new UICommand()
            {
                Caption  = "Add",
                IsCancel = false,
                IsDefault = true
            };

            UICommand cancelProductUICommand = new UICommand()
            {
                Caption   = "Cancel",
                IsDefault = false,
                IsCancel  = true
            };

            UICommand resultUICommand = DialogService.ShowDialog(new List<UICommand>() { addProductUICommand, cancelProductUICommand}, "Add New Product", product);

            if(resultUICommand == addProductUICommand)
            {
                this.productCollection.Add(product);
            }
        }

        #endregion
    }
}